package com.wonders.plugin;

import java.util.List;

import javax.servlet.ServletException;

import net.sf.navigator.menu.MenuComponent;
import net.sf.navigator.menu.MenuRepository;
import net.sf.navigator.util.LoadableResourceException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionServlet;
import org.apache.struts.action.PlugIn;
import org.apache.struts.config.ModuleConfig;

import com.wonders.dao.Menu;
import com.wonders.global.Constants;
import com.wonders.service.IService;
import com.wonders.service.IServiceFactory;
/**
 * <p><strong>ServiceFactory</strong>是一个工厂类,用于产生<strong>INetService</strong>接口.
 * 它同时实现了<code>INetServiceFactory</code>和<code>PlugIn</code>接口,通过在web.xml中配置
 * service_class参数，关键的方法是<code>getINetServie</code></p>
 * 
 * <p>
 * web.xml中的参数可以如下配置：
 * <pre>
 * <plugin class="com.wonders.plugin.ServiceFactoryPlugin">
 * <set-property property="serviceConfig" value="com.wonders.service.ServiceImpl"/>
 * </plugin>
 * </pre>
 * </p>
 * @author <a href="mailto:yanming3@hotmail.com">yanming3</a>
 *
 */
public class ServiceFactoryPlugin implements IServiceFactory,PlugIn{
	private Log log=LogFactory.getLog(ServiceFactoryPlugin.class);
	private String serviceConfig="com.wonders.service.ServiceImpl";
	private MenuRepository repository=null;
    private String menuConfig = "/WEB-INF/menu-config.xml";
	private ActionServlet servlet;
	
	 public String getMenuConfig() {
	        return menuConfig;
	    }

	    public void setMenuConfig(String menuConfig) {
	        this.menuConfig = menuConfig;
	    }
	    
	public void destroy() {
		this.servlet=null;
	}
	/**
	 * <p>读取配置文件定义</p>
	 */
	public void init(ActionServlet servlet, ModuleConfig module) throws ServletException {
		this.servlet=servlet;
		if((serviceConfig==null)||serviceConfig.equals("")){
			serviceConfig="com.wonders.service.ServiceImpl";}
	
		servlet.getServletContext().setAttribute(Constants.FACTORY_CLASS, this);
		
		if (log.isDebugEnabled()) {
            log.debug("Starting struts-menu initialization");
        }

        repository = new MenuRepository();
        repository.setLoadParam(menuConfig);
        repository.setServletContext(servlet.getServletContext());

        try {
            repository.load();
            addMenu();
            //servlet.getServletContext().setAttribute(MenuRepository.MENU_REPOSITORY_KEY, repository);
            servlet.getServletContext().setAttribute("repository", repository);
            if (log.isDebugEnabled()) {
                log.debug("struts-menu initialization successful");
            }
        } catch (LoadableResourceException lre) {
            throw new ServletException("Failure initializing struts-menu: " +
                lre.getMessage());
        }
	}
	/**
	<p>获取<code>INetService</code>对象，程序会先读取配置文件中的定义，若不存在，对使用默认值．</p>
	*/
	public IService getINetService() {
		IService ins=null;
		try{
			ins=(IService)Class.forName(serviceConfig).newInstance();
			
            }
			catch(ClassNotFoundException e){
				log.debug("指定的类不存在",e);
			}
			catch(IllegalAccessException f){
				log.debug("无法访问指定类、字段、方法或构造方法的定义",f);
			}
			catch(InstantiationException g){
				log.debug("指定的类对象是一个接口或是一个抽象类而无法实例化",g);
			}
			ins.setServletContext(servlet.getServletContext());
			return ins;
	}
	public void addMenu(){
    	IService service=getINetService();
    	List menuList=service.getAllMenu();
    	
    	for (int i=0; i < menuList.size(); i++) {
    	    MenuComponent mc = new MenuComponent();
    	    Menu menu=(Menu)menuList.get(i);
    	    mc.setName(menu.getName());
    	    String parent=menu.getParentName();
    	    if (parent != null) {
    	        MenuComponent parentMenu = repository.getMenu(parent);
    	        /**
    	         * 如果父菜单不存在,创建一个临时的父菜单
    	         */
    	        if (parentMenu == null) {
    	            parentMenu = new MenuComponent();
    	            parentMenu.setName(parent);
    	            repository.addMenu(parentMenu);
    	        }

    	        mc.setParent(parentMenu);
    	    }
    	    mc.setTitle(menu.getTitle());
    	    mc.setForward(menu.getForward());
    	    mc.setLocation(menu.getLocation());
    	    mc.setAction(menu.getAction());
    	    repository.addMenu(mc);
    	}
    }

	public String getServiceConfig() {
		return serviceConfig;
	}

	public void setServiceConfig(String serviceConfig) {
		this.serviceConfig = serviceConfig;
	}
}
